unit main;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, SpiderCGI, process, SpiderUtils, SpiderTable,
  SpiderApache, db;

type

  { TDataModule1 }

  TDataModule1 = class(TDataModule)
    Process1: TProcess;
    SpiderApache1: TSpiderApache;
    SpiderCGI1: TSpiderCGI;
    SpiderTable1: TSpiderTable;
    stPartitions: TSpiderTable;
    procedure DataModuleCreate(Sender: TObject);
    procedure SpiderCGI1Request(Sender: TObject; Request: TSpiderRequest;
      var Response: TSpiderResponse);
    procedure stPartitionsDrawDataCell(Sender: TObject; DataSet: TDataSet;
      ColCount, RowCount: Integer; var CellData, BgColor, ExtraParams: string);
  private
    { private declarations }
    function TextToHTML(AText: string): string;
    function GetParitions: string;
    function GetMemoryInfo: string;
    function GetDistroInfo: string;
    function GetArchetInfo: string;
  public
    function ShellCommand(Command: string; var ResultStr: string): Boolean;
    { public declarations }
  end;

var
  DataModule1: TDataModule1;

implementation


{$R *.lfm}


{ TDataModule1 }

procedure TDataModule1.DataModuleCreate(Sender: TObject);
begin
  SpiderCGI1.Execute;
end;

procedure TDataModule1.SpiderCGI1Request(Sender: TObject;
  Request: TSpiderRequest; var Response: TSpiderResponse);
var
  Result: string;
begin
  Response.Add('<html><header><title>Linux Window</title>');
  Response.Add('<style>th {font-size:13px; font-family:Arial; font-style:normal;}');
  Response.Add('td {font-size:12px;}');
  Response.Add('p {font-size:13px; font-family:DejaVu Sans Mono; font-style:normal;}');
  Response.Add('h3 {font-family:DejaVu Sans Mono; color:#605555;}');
  Response.Add('</style><body>');
  SpiderTable1.SetHeader(['', '']);

  Response.Add('<h3>Linux Window</h3>');
  ShellCommand('hostname', Result);
  SpiderTable1.AddRow('', ['Host name ', '<b>' + Result + '</b>']);
  SpiderTable1.AddRow('', ['Host Time ', '<b>' + FormatDateTime('yyyy-mm-dd hh:nn', Now) + '</b>']);

  ShellCommand('who -b', Result);
  SpiderTable1.AddRow('', ['Boot time ', '<b>' + Result + '</b>']);

  Response.Add(SpiderTable1.Contents);

  Response.Add('<br/>');

  Response.Add('<h3>Distribution Info</h3>');
  Response.Add('<p>' + GetDistroInfo);
  Response.Add(TextToHTML('Arch          : ' + GetArchetInfo));
  Response.Add('</p><br/>');


  Response.Add('<h3>File System</h3>');
  Response.Add(GetParitions);

  Response.Add('<br/>');

  Response.Add('<h3>Memory</h3>');
  Response.Add(GetMemoryInfo);

  Response.Add('</body></html>');

end;

procedure TDataModule1.stPartitionsDrawDataCell(Sender: TObject;
  DataSet: TDataSet; ColCount, RowCount: Integer; var CellData, BgColor,
  ExtraParams: string);
var
  Per: Integer;
begin
  if ColCount = 4 then
  begin
    Per:= StrToInt(Copy(CellData, 1, Length(CellData) - 1));

    CellData:= IntToStr(Per);
    if Per > 95 then
      BgColor:= '#FF5555'
    else
    if Per > 90 then
      BgColor:= '#FF9999'
    else
      if Per > 50 then
        BgColor:= '#EAEA62'
      else
        if Per > 0 then
        BgColor:= '#AAFFAA';
  end;
end;

function TDataModule1.TextToHTML(AText: string): string;
begin
  Result:= StringReplace(AText, #10, '<br/>', [rfReplaceAll]);
  Result:= StringReplace(Result, ' ', '&emsp;', [rfReplaceAll]);
end;

function TDataModule1.GetParitions: string;
var
  ResultStr: string;
  List: TStringList;
  RowList: TStringList;
  i: Integer;
begin
  ShellCommand('df -h', ResultStr);
  ResultStr:= StringReplace(ResultStr, 'Mounted on', 'Mounted-on', []);
  List:= TStringList.Create;
  while ResultStr <> '' do
  begin
    List.Add(Copy(ResultStr, 1, Pos(#10, ResultStr)));
    System.Delete(ResultStr, 1, Pos(#10, ResultStr));
    ResultStr:= Trim(ResultStr);
    if (ResultStr <> '') and (Pos(#10, ResultStr) = 0) then
      ResultStr:= ResultStr + #10;
  end;
  RowList:= TStringList.Create;
  if List.Count > 0 then
  begin;
    RowList.CommaText:= List[0];
    stPartitions.HeaderColor:= '#FBEEFF';
    stPartitions.SetHeader(RowList);
    for i:= 1 to List.Count - 1 do
    begin
      RowList.CommaText:= List[i];
      stPartitions.AddRow(RowList, '#FBFEFE');
    end;
    Result:= stPartitions.Contents;

  end
  else
    Result:= List.Text;
  List.Free;
  RowList.Free;

end;

function TDataModule1.GetMemoryInfo: string;
var
  Line: string;
  MemTotal, MemFree: Integer;
  MemFreeF: Double;
  Per: Double;
begin
  ShellCommand('grep Mem  /proc/meminfo', Result);
  Line:= Copy(Result, 1, Pos(#10, Result));
  Line:= Copy(Line, Pos(':', Line) + 1, Length(Line));
  Line:= Trim(Line);
  Line:= Copy(Line, 1, Pos(' ', Line) - 1);
  MemTotal:= StrToInt(Line);
  Delete(Result, 1, Pos(#10, Result));
  Line:= Copy(Result, Pos(':', Result) + 1, Length(Result));
  Line:= Trim(Line);
  Line:= Copy(Line, 1, Pos(' ', Line) - 1);
  MemFree:= StrToInt(Line);
  Result:= '<p>Memory Size ' + Format('%3.2n', [((MemTotal / 1024) / 1024) ]) + ' Gigabytes<br/>';
  MemFreeF:= ((MemFree / 1024) / 1024);
  Result:= Result + '<p>Free Size &emsp;&emsp;<font color=';

  Per:= (MemFree / MemTotal) * 100;
  if Per < 10 then
    Result:= Result + 'red>'
  else
    Result:= Result + 'green>';

  Result:= Result + Format('%3.2n', [MemFreeF]) + '</font> Gigabytes<br/>';

end;

function TDataModule1.GetDistroInfo: string;
var
  ResultStr: string;
  Rec: TSearchRec;
begin
  //if ShellCommand('lsb_release -a', ResultStr) then
  if FindFirst('/etc/*-release', faAnyFile, Rec) = 0 then
  if ShellCommand('cat /etc/' + Rec.Name, ResultStr) then
  begin
    Result:= TextToHTML(ResultStr);
  end
  else
    Result:= 'Error';
end;

function TDataModule1.GetArchetInfo: string;
begin
  ShellCommand('uname -m', Result);
end;

function TDataModule1.ShellCommand(Command: string; var ResultStr: string): Boolean;
var
  List: TStringList;
begin
  try
    Process1.CommandLine:= Command;
    Process1.Execute;
    List:= TStringList.Create;
    List.LoadFromStream(Process1.Output);
    ResultStr:= List.Text;
    Result:= True;
    List.Free;

  except
  on e: exception do
  begin
    Result:= False;
    ResultStr:= e.message;
  end;
  end;
end;

end.

